-- #####################################################################################
-- ## Skript zum Workshop <SQLBattleShips>                                            ##     
-- #####################################################################################
-- ## Dieses Skript legt Datenbankobjekte fuer den Workshop <SQLBattleShips> an. Es   ##
-- ## wird durch weitere Skripte ergaenzt, die in ihrer durchnummerierten Reihenfolge ##
-- ## abzuarbeiten sind. Die Skripte passen zu den Ausfuehrungen aus der zugehoerigen ##
-- ## Powerpoint-Praesentation vom selben Autor.                                      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergaenzt, geaendert oder sogar geloescht werden!!!   ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf die Workshops-Datenbank wechseln
USE [Vortrag_Kopfnuss]
GO

-- Gibt an, dass sich die Vergleichsoperatoren Gleich (=) und Ungleich (<>) bei Verwendung mit NULL-Werten in SQL Server 2019 (15.x) ISO-konform verhalten muessen.
-- ANSI NULLS ON ist neuer T-SQL Standard und wird in spaeteren Versionen festgeschrieben.
SET ANSI_NULLS ON
GO

-- Bewirkt, dass SQL Server die ISO-Regeln fuer Anfuehrungszeichen bei Bezeichnern und Literalzeichenfolgen befolgt.
SET QUOTED_IDENTIFIER ON
GO






--------------------------------------------------------------------------------------------------
-- Statistiken -----------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------

-- temporaere Tabelle anlegen, um sich die Startzeit zu merken
BEGIN TRY
	DROP TABLE #Start
END TRY
BEGIN CATCH
END CATCH

CREATE TABLE #Start (StartTime DATETIME)
INSERT INTO #Start (StartTime) VALUES (GETDATE())














-- ######################################################################################
-- ### Tabelleninhalte loeschen, um sie spaeter sauber wieder einspielen zu koennen   ###
-- ######################################################################################
TRUNCATE TABLE [BattleShips].[ListeEigenesSpielbrett]
TRUNCATE TABLE [BattleShips].[ListeEigenesSchussfeld]
GO





-- ######################################################################################
-- ### Tabelleninhalte einspielen                                                     ###
-- ######################################################################################


-- Nun gilt es vier Tabellen ([ListeEigenesSpielbrett] und [ListeEigenesSchussfeld] fr jeweils beide Spieler) mit einem geschtzten Leerzeichen 
-- als [Inhalt] und dem Wert NULL als [Zugnummer] zu fllen. Die Felder [Zielkoordinate], [Reihe] bzw. [Spalte] sollen dabei automatisch
-- in einer Schleife hochgezhlt werden. Da es sich teilweise um Buchstaben (Spaltennamen: A-J) handelt, ist die Funktion CHAR(x) zu nutzen. 
-- Hilfreich ist es an der Stelle, sich mit der ASCII-Tabelle
DECLARE @SchleifenZaehlerZeilen			INTEGER
DECLARE @SchleifenZaehlerSpalten		INTEGER
DECLARE @Zielfeld						CHAR(3)

SET @SchleifenZaehlerZeilen				= 1

WHILE @SchleifenZaehlerZeilen <= 10
BEGIN
	SET @SchleifenZaehlerSpalten		= 65			-- CHAR(65) entspricht einem 'A'
	WHILE @SchleifenZaehlerSpalten		<= 74			-- CHAR(74) entspricht einem 'J'
	BEGIN
		SET @Zielfeld = CHAR(@SchleifenZaehlerSpalten) + CONVERT(VARCHAR(2), @SchleifenZaehlerZeilen)
		
		-- Bestuecke das Spielbrett 
		INSERT INTO [BattleShips].[ListeEigenesSpielbrett] ([Zielkoordinate], [Reihe], [Spalte], [Inhalt], [Anzeige], [ZugnummerGegner])
					VALUES (
						CHAR(@SchleifenZaehlerSpalten) + CONVERT(VARCHAR(2), @SchleifenZaehlerZeilen)
						,  @SchleifenZaehlerZeilen
						, CHAR(@SchleifenZaehlerSpalten)
						, ' '
						, ' '
						, NULL)
		-- Bestuecke das Schussfeld 
		INSERT INTO [BattleShips].[ListeEigenesSchussfeld] ([Zielkoordinate], [Reihe], [Spalte], [Inhalt], [Anzeige], [EigeneZugnummer])
					VALUES (
						CHAR(@SchleifenZaehlerSpalten) + CONVERT(VARCHAR(2), @SchleifenZaehlerZeilen)
						,  @SchleifenZaehlerZeilen
						, CHAR(@SchleifenZaehlerSpalten)
						, ' '
						, ' '
						, NULL)		

		SET @SchleifenZaehlerSpalten	= @SchleifenZaehlerSpalten + 1
	END
	SET @SchleifenZaehlerZeilen			= @SchleifenZaehlerZeilen + 1
END
GO



UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'F', [Anzeige] = 'F' 
WHERE [Zielkoordinate] IN ('B2', 'B3', 'B4', 'B5', 'B6')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'S', [Anzeige] = 'S' 
WHERE [Zielkoordinate] IN ('D3', 'E3', 'F3', 'G3')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'K', [Anzeige] = 'K' 
WHERE [Zielkoordinate] IN ('J7', 'J8', 'J9')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'T', [Anzeige] = 'T' 
WHERE [Zielkoordinate] IN ('I1', 'I2', 'I3')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'M', [Anzeige] = 'M' 
WHERE [Zielkoordinate] IN ('H6')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = ' ', [Anzeige] = 'o' 
WHERE [Zielkoordinate] IN ('A1', 'D1', 'A7', 'D6', 'E5', 'G9', 'E10', 'H10')




UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'U', [Anzeige] = 'x' 
WHERE [Zielkoordinate] IN ('C8', 'D8')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'S', [Anzeige] = 'x' 
WHERE [Zielkoordinate] IN ('E3', 'F3')

UPDATE [BattleShips].[ListeEigenesSpielbrett]
SET [Inhalt] = 'K', [Anzeige] = 'x' 
WHERE [Zielkoordinate] IN ('J9')

-----------------------

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Inhalt] = 'F', [Anzeige] = 'F' 
WHERE [Zielkoordinate] IN ('C3', 'C4', 'C5', 'C6', 'C7')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Inhalt] = 'S', [Anzeige] = ' ' 
WHERE [Zielkoordinate] IN ('B1', 'C1', 'D1', 'E1')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Inhalt] = 'K', [Anzeige] = ' ' 
WHERE [Zielkoordinate] IN ('H2', 'H3', 'H4')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Inhalt] = 'U', [Anzeige] = ' ' 
WHERE [Zielkoordinate] IN ('E6', 'F6')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Inhalt] = 'T', [Anzeige] = ' ' 
WHERE [Zielkoordinate] IN ('A7', 'A8', 'A9')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Anzeige] = 'o' 
WHERE [Zielkoordinate] IN ('C2', 'C8', 'B2', 'B3', 'B4', 'B5', 'B6', 'B7', 'B8','D2', 'D3', 'D4', 'D5', 'D6', 'D7', 'D8')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Inhalt] = 'K', [Anzeige] = 'x' 
WHERE [Zielkoordinate] IN ('H3', 'H4')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Anzeige] = 'o' 
WHERE [Zielkoordinate] IN ('H6', 'J7')

UPDATE [BattleShips].[ListeEigenesSchussfeld]
SET [Anzeige] = 'o' 
WHERE [Zielkoordinate] IN ('A1', 'A6', 'F4', 'F5', 'G2')



------------------------------------------------------------------------------------------------------------------------------------------------------
-- Statistiken ---------------------------------------------------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------------------------------------------------------------------
DECLARE @StartTime	DATETIME		= (SELECT StartTime FROM #Start)
DECLARE @Ende		VARCHAR(25)		= CONVERT(VARCHAR(25), GETDATE(), 104) + '   ' +CONVERT(VARCHAR(25), GETDATE(), 114)
DECLARE @Zeit		VARCHAR(500)	= CAST(DATEDIFF(SS, @StartTime, GETDATE()) AS VARCHAR(10)) + ',' + CAST(DATEPART(MS, GETDATE() - @StartTime) AS VARCHAR(10)) + ' sek.'
DECLARE @Skript		VARCHAR(100)	= '032 - Tabelle(n) fuellen - [Spielbrett], [Schussfeld].sql'
PRINT ' '
PRINT 'Skript     :   ' + @Skript
PRINT 'Ende       :   ' + @Ende
PRINT 'Zeit       :   ' + @Zeit
SELECT @Skript AS Skript, @Ende AS Ende, @Zeit AS Zeit
GO
