-- #####################################################################################
-- ## Skript zum Vortrag <die schwierigen Kopfnsse mit T-SQL gelst - Version 2.0>   ##     
-- #####################################################################################
-- ## Dieses Skript legt die Projektdatenbank an. Es wird durch weitere Skripte       ##
-- ## ergnzt, die in ihrer durchnummerierten Reihenfolge abzuarbeiten sind. Die      ##
-- ## Skripte passen zu den Aufgabenstellungen aus der zugehrigen Powerpoint-        ##
-- ## Prsentation vom selben Autor.                                                  ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- "richtige" Datenbank auswhlen
USE [Workshop_Kopfnuesse]
GO

-----------------------------
-- Aufrumarbeiten ----------
-----------------------------
-- bestehende View vZufallszahl lschen, um sauber wieder aufzusetzen
IF OBJECT_ID('[Sudoku].[vZufallszahl]', 'V') IS NOT NULL
	DROP VIEW [Sudoku].[vZufallszahl];
GO

IF OBJECT_ID('[Sudoku].[fnZufall]', 'FN') IS NOT NULL
	DROP FUNCTION [Sudoku].[fnZufall];
GO

IF OBJECT_ID('[Sudoku].[spSudokuErstellen]', 'P') IS NOT NULL
	DROP PROCEDURE [Sudoku].[spSudokuErstellen];
GO

IF OBJECT_ID('[Sudoku].[spSudokuLoesen]', 'P') IS NOT NULL
	DROP PROCEDURE [Sudoku].[spSudokuLoesen];
GO

DROP SCHEMA [Sudoku]
GO

-----------------------------
-- Aufbauarbeiten ----------
-----------------------------
-- Schema Sudoku anlegen, um Objekte sauber der Aufgabe zuordnen zu knnen
CREATE SCHEMA [Sudoku]
GO

-- View vZufallszahl erstellen
CREATE VIEW [Sudoku].[vZufallszahl]
AS
  SELECT RAND() AS RandomZiffer;
GO
 
-- Diese Funktion gibt eine Zufallszahl im Bereich von @Untergrenze bis @Obergrenze zurck
CREATE or alter FUNCTION [Sudoku].[fnZufall]
  (
    @Untergrenze			INTEGER
  , @Obergrenze				INTEGER
  )
RETURNS INT
AS
  BEGIN
  RETURN 
   FLOOR((SELECT [Sudoku].[vZufallszahl].[RandomZiffer] FROM [Sudoku].[vZufallszahl])
                * (@Untergrenze - @Obergrenze + 1) + @Untergrenze
        );
  END;
GO
 

 
-- Hier wird ein Sudokufeld regelkonform gefllt
CREATE or alter PROCEDURE [Sudoku].[spSudokuErstellen]
AS
-- Die Tabellenwertvariable @ErlaubteWerte soll die erlaubten Werte, also die Ziffern 1-9, annehmen
DECLARE @ErlaubteWerte TABLE(Ziffer INT);
	INSERT INTO @ErlaubteWerte SELECT 1;
	INSERT INTO @ErlaubteWerte SELECT 2;
	INSERT INTO @ErlaubteWerte SELECT 3;
	INSERT INTO @ErlaubteWerte SELECT 4;
	INSERT INTO @ErlaubteWerte SELECT 5;
	INSERT INTO @ErlaubteWerte SELECT 6;
	INSERT INTO @ErlaubteWerte SELECT 7;
	INSERT INTO @ErlaubteWerte SELECT 8;
	INSERT INTO @ErlaubteWerte SELECT 9;
 

DECLARE @SudokuMatrix TABLE (
    [ID]				INTEGER IDENTITY(1, 1)
  , [Spalte]			INTEGER
  , [Zeile]				INTEGER
  , [Region]			INTEGER
  , [Wert]				INTEGER
  );

INSERT INTO @SudokuMatrix
  ( [Spalte]
  , [Zeile]
  , [Region]
  , [Wert])
  SELECT 
	  [Spalte].[Ziffer]																-- Spalte
	, [Zeile].[Ziffer]																-- Zeile
	, (([Zeile].[Ziffer] - 1) / 3) * 3 + (([Spalte].[Ziffer] - 1) / 3) + 1			-- Region
	, NULL																			-- Wert (Inhalt)
  FROM @ErlaubteWerte [Spalte]
    CROSS JOIN @ErlaubteWerte [Zeile];
 

DECLARE @MoeglicheWerte TABLE
  (
    [ID]				INTEGER IDENTITY(1, 1)
  , [Wert]				INTEGER
  , [Spalte]			INTEGER
  , [Zeile]				INTEGER
  , [Region]			INTEGER
  , [bereitsGenutzt]	INTEGER
  );

INSERT INTO @MoeglicheWerte
	(   [Zeile]
	  , [Spalte]
	  , [Region]
	  , [Wert]
	  , [bereitsGenutzt]
	)
  SELECT 
	    [@SudokuMatrix].[Spalte]
	  ,	[@SudokuMatrix].[Zeile]
	  , [@SudokuMatrix].[Region]
	  , [@ErlaubteWerte].[Ziffer]
	  , 0
  FROM @SudokuMatrix
	CROSS JOIN @ErlaubteWerte;



 
DECLARE @ZeileImSet		INTEGER;
DECLARE @SpalteImSet	INTEGER;
DECLARE @RegiontoSet	INTEGER;
DECLARE @Wert			INTEGER;
 
-- Schleife, bis eine gltige Matrix heraus kommt
WHILE EXISTS (SELECT 1 FROM @SudokuMatrix WHERE [@SudokuMatrix].Wert IS NULL)
BEGIN
	UPDATE @SudokuMatrix		SET [@SudokuMatrix].[Wert]				= NULL;
	UPDATE @MoeglicheWerte		SET [@MoeglicheWerte].[bereitsGenutzt]	= 0;
	WHILE EXISTS (
				SELECT 1 FROM @MoeglicheWerte WHERE [@MoeglicheWerte].[bereitsGenutzt] = 0
				)
	BEGIN
		-- Suche alle Stellen, die nur noch einen mglichen Wert aufnehmen knnen
		SELECT TOP 1
			  @SpalteImSet		= [@MoeglicheWerte].[Spalte]
			, @ZeileImSet		= [@MoeglicheWerte].[Zeile]
			, @RegiontoSet		= MAX([@MoeglicheWerte].Region)
		FROM @MoeglicheWerte
		WHERE 1=1
			AND [@MoeglicheWerte].[bereitsGenutzt] = 0
		GROUP BY 
			  [@MoeglicheWerte].[Spalte]
			, [@MoeglicheWerte].[Zeile]
		HAVING COUNT(*) = 1;
		
		IF @@RowCount = 0
		-- finde die Stelle, die noch die meisten Mglichkeiten bietet
		SELECT TOP 1 
			  @SpalteImSet		= [@MoeglicheWerte].[Spalte]
			, @ZeileImSet		= [@MoeglicheWerte].[Zeile]
			, @RegiontoSet		= MAX([@MoeglicheWerte].[Region])
		FROM @MoeglicheWerte
		WHERE 1=1
			AND [@MoeglicheWerte].[bereitsGenutzt] = 0
		GROUP BY 
			  [@MoeglicheWerte].[Spalte]
			, [@MoeglicheWerte].[Zeile]
		ORDER BY 
			  COUNT(*) ASC
			, [Sudoku].[fnZufall](1, 20) DESC;

		-- zufllig eine Zahl whlen
		SELECT TOP 1 
			@Wert = [@MoeglicheWerte].Wert
		FROM @MoeglicheWerte
		WHERE 1=1 
			AND [@MoeglicheWerte].[Spalte]			= @SpalteImSet
			AND [@MoeglicheWerte].[Zeile]			= @ZeileImSet
			AND [@MoeglicheWerte].[bereitsGenutzt]	= 0
		ORDER BY [Sudoku].[fnZufall](2, 6);
		
		-- sicher stellen, dass diese Stelle keine weiteren Mglichkeiten aufweist
		UPDATE @MoeglicheWerte
		SET [@MoeglicheWerte].[bereitsGenutzt] = 1
		WHERE 1=1
			AND [@MoeglicheWerte].[Spalte]			= @SpalteImSet
			AND [@MoeglicheWerte].[Zeile]			= @ZeileImSet;

		-- den Gebrauch in der Zeile markieren
		UPDATE @MoeglicheWerte
		SET [@MoeglicheWerte].[bereitsGenutzt] = 1
		WHERE 1=1
			AND [@MoeglicheWerte].Wert				= @Wert
			AND [@MoeglicheWerte].[Zeile]			= @ZeileImSet;

		-- den Gebrauch in der Spalte markieren
		UPDATE @MoeglicheWerte
		SET [@MoeglicheWerte].[bereitsGenutzt] = 1
		WHERE 1=1
			AND [@MoeglicheWerte].Wert				= @Wert
			AND [@MoeglicheWerte].[Spalte]			= @SpalteImSet;

		-- den Gebrauch im Unterquadrat markieren
		UPDATE @MoeglicheWerte
		SET [@MoeglicheWerte].[bereitsGenutzt] = 1
		WHERE 1=1
			AND [@MoeglicheWerte].Wert				= @Wert
			AND [@MoeglicheWerte].Region			= @RegiontoSet;

		-- gefundene (Teil-)Lsung in die Matrix bernehmen
 		UPDATE @SudokuMatrix
		SET [@SudokuMatrix].Wert					= @Wert
		WHERE 1=1
			AND [@SudokuMatrix].[Spalte]			= @SpalteImSet
			AND [@SudokuMatrix].[Zeile]				= @ZeileImSet;
	END;
END;
 
SELECT 
	  [a]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=1 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [b]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=2 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [c]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=3 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [d]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=4 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [e]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=5 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [f]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=6 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [g]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=7 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [h]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=8 THEN [@SudokuMatrix].Wert ELSE 0 END))
	, [i]=CONVERT(INT,MAX(CASE WHEN [@SudokuMatrix].[Spalte]=9 THEN [@SudokuMatrix].Wert ELSE 0 END))
FROM  @SudokuMatrix GROUP BY [@SudokuMatrix].[Zeile]




--SELECT [@SudokuMatrix].Wert FROM  @SudokuMatrix 