-- #####################################################################################
-- ## Skript zum Vortrag <die schwierigen Kopfnsse mit T-SQL gelst>                 ##     
-- #####################################################################################
-- ## Dieses Skript legt die Tabellen fr die Aufgabe T-SQL-Mine an. Es wird durch    ##
-- ## weitere Skripte ergnzt, die in ihrer durchnummerierten Reihenfolge abzu-       ##
-- ## arbeiten sind. Die Skripte passen zu den Ausfhrungen aus der zugehrigen       ##
-- ## Powerpoint-Prsentation vom selben Autor.                                       ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- ## Nach einer Idee von Auke Teeninga im Simple-Talk von Redgate                    ##
-- ## https://www.red-gate.com/simple-talk/sql/t-sql-programming/minesweeper-in-t-sql/##
-- #####################################################################################

-- auf die Projektdatenbank wechseln
USE [Workshop_Kopfnuesse]
GO

-- ----------------------------------------------------------------------------------
-- Prfen, ob es das Schema [TSQLMine] schon gibt. 
-- Falls nicht, dann wird es angelegt
-- ----------------------------------------------------------------------------------
IF NOT EXISTS 
	(
		SELECT  schema_name
		FROM    information_schema.schemata
		WHERE   schema_name = 'TSQLMine' 
	)
	BEGIN
		EXEC SP_EXECUTESQL N'CREATE SCHEMA [TSQLMine]'
	END
GO

-- ----------------------------------------------------------------------------------
-- Falls es die jeweilige Tabelle schon gibt, wird diese entfernt
-- ----------------------------------------------------------------------------------
DROP TABLE IF EXISTS [TSQLMine].[Spielfeld]
GO
DROP TABLE IF EXISTS [TSQLMine].[Konfiguration]
GO


-- ----------------------------------------------------------------------------------
-- Anlage der Tabelle [TSQLMine].[Spielfeld]
-- ----------------------------------------------------------------------------------
CREATE TABLE [TSQLMine].[Spielfeld]
    (
		  [XKoord]												INT			NOT NULL                                                                                          -- X-Koordinate
		, [YKoord]												INT			NOT NULL                                                                                          -- Y-Koordinate
		, [Status]												NCHAR(1)	NOT NULL                                                                                          -- Aktueller Feld-Status (Ansicht), 
			CHECK ([Status] IN ('?', '0', '1', '2', '3', '4') OR					-- Buchstabe fr: 
				   [Status] IN ('5', '6', '7', '8', '9', 'X'))						--   '?'		= verdeckt, unbekannt
																					--   'X'		= markiert als Bombenverdacht
																					--   Ziffern	= Anzahl Bomben in direkter Umgebung
		, [Inhalt]                                              NCHAR(1)	NULL                                                                                                    -- Tatschlicher (evtl. noch verdeckter) Inhalt:
			--CHECK ([Inhalt] IS NULL OR
			--	   [Inhalt] IN ('M', '0', '1', '2', '3', '4') OR					-- Buchstabe fr: 
			--	   [Inhalt] IN ('5', '6', '7', '8', '9'))							--   NULL		= keine Mine 
                                                                                    --   'M'		= hier wurde eine Mine platziert
CONSTRAINT PK_Spielfeld PRIMARY KEY ([XKoord] ASC, [YKoord] ASC)
    )
GO


-- ----------------------------------------------------------------------------------
-- Anlage der Tabelle [TSQLMine].[Konfiguration]
-- ----------------------------------------------------------------------------------
CREATE TABLE [TSQLMine].[Konfiguration]
    (
		  [XMax]			INT		NOT NULL										-- X-Ausdehnung des Spielfeldes
		, [YMax]			INT		NOT NULL										-- Y-Ausdehnung des Spielfeldes
		, [AnzahlMinen]		INT		NOT NULL										-- Anzahl der zu findenen Minen
    )
GO

-- ----------------------------------------------------------------------------------
-- Einfgen einer Default-Werte
-- ----------------------------------------------------------------------------------
INSERT  INTO [TSQLMine].[Konfiguration]
	VALUES  (12, 12, 20)
GO


-- ----------------------------------------------------------------------------------
-- Die Prozedur [TSQLMine].[prcInitialisierung] berschreibt die Default-Werte 
-- in der Konfigurationstabelle, falls neue Werte angegeben werden. Anschlieend 
-- werden nach dieser Konfiguration zufllig Minen verteilt
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcInitialisierung]
    (
          @XMax			INT = NULL
        , @YMax			INT = NULL
        , @AnzahlMinen	INT = NULL
        , @Seed			INT = NULL
    )
AS
    BEGIN
        -- Zuerst die Konfiguration aktualisieren
        UPDATE  [TSQLMine].[Konfiguration]
        SET	  XMax				= ISNULL(@XMax, XMax)
			, YMax				= ISNULL(@YMax, YMax)
			, AnzahlMinen		= ISNULL(@AnzahlMinen, AnzahlMinen)
           
        -- die Konfiguration in die Variablen bernehmen
        SELECT	  @XMax			= XMax
				, @YMax			= YMax
				, @AnzahlMinen	= AnzahlMinen
        FROM [TSQLMine].[Konfiguration]

        -- Tabelle vorsichthalber (vorangegangene Spiele) leeren
        TRUNCATE TABLE [TSQLMine].[Spielfeld]

        DECLARE @XKoord INT
        DECLARE @YKoord INT

		-- Spielfeld aufbauen. Dazu ber ineinandergeschachtelte Schleifen mit der 
		-- X- und Y-Ausdehnung Felder aneinanderreihen und mit "?" (= "unbekannt") vorbelegen
        SET @XKoord = 1
        WHILE @XKoord <= @XMax
            BEGIN
                SET @YKoord = 1
                WHILE @YKoord <= @YMax
                    BEGIN
                        INSERT  INTO [TSQLMine].[Spielfeld]
                        VALUES  (@XKoord, @YKoord, '?', NULL)
                        SET @YKoord = @YKoord + 1
                    END
                SET @XKoord = @XKoord + 1
            END

        -- Die Zufallskomponente fr die Minenplatzierung initialisieren
        IF @seed IS NOT NULL SET @XKoord = RAND(@seed)

        -- fr jede konfigurierte Mine eine (neue) Koordinate suchen
        WHILE @AnzahlMinen > 0
        BEGIN
			SET @XKoord = FLOOR(RAND() * @XMax) + 1
			SET @YKoord = FLOOR(RAND() * @YMax) + 1
			-- Falls Koordinate schon belegt, Versuch wiederholen - sonst
			-- Mine speichern und Minenzhler verringern
			IF (SELECT  COUNT([Inhalt])
				FROM [TSQLMine].[Spielfeld]
				WHERE 1 = 1
						AND [XKoord] = @XKoord
						AND [YKoord] = @YKoord
						AND [Inhalt] = 'M'
				) = 0
			BEGIN
				UPDATE [TSQLMine].[Spielfeld] 
				SET [Inhalt] = 'M'
				WHERE 1 = 1
						AND [XKoord] = @XKoord
						AND [YKoord] = @YKoord
				SET @AnzahlMinen = @AnzahlMinen - 1
			END
        END 

        -- Spielfeld aufbauen. Dazu ber ineinandergeschachtelte Schleifen mit der 
        -- X- und Y-Ausdehnung Felder aneinanderreihen und mit "?" (= "unbekannt") vorbelegen
        SET @XKoord = 1
        WHILE @XKoord <= @XMax
        BEGIN
            SET @YKoord = 1
            WHILE @YKoord <= @YMax
                BEGIN
                    UPDATE [TSQLMine].[Spielfeld]
                    SET [Inhalt] = CONVERT(CHAR(1), ISNULL(
                        (
                            SELECT COUNT(*)
                            FROM   [TSQLMine].[Spielfeld] g1
                            INNER JOIN [TSQLMine].[Spielfeld] g2 ON    1 = 1
                            WHERE 1 = 1
                                    AND [g2].[Inhalt] = 'M'                                            -- Mine gefunden
                                    AND [g1].[XKoord] = @XKoord
                                    AND [g1].[YKoord] = @YKoord
									AND ([g1].[XKoord] = [g2].[XKoord] + 1 OR [g1].[XKoord] = [g2].[XKoord] - 1 OR [g1].[XKoord] = [g2].[XKoord])
									AND ([g1].[YKoord] = [g2].[YKoord] + 1 OR [g1].[YKoord] = [g2].[YKoord] - 1 OR [g1].[YKoord] = [g2].[YKoord])
									AND NOT ([g1].[XKoord] = [g2].[XKoord] AND [g1].[YKoord] = [g2].[YKoord])
                        ), 0))
                    WHERE 1 = 1
                        AND [XKoord] = @XKoord
                        AND [YKoord] = @YKoord
                        AND [Inhalt] IS NULL
                    SET @YKoord = @YKoord + 1
                END
            SET @XKoord = @XKoord + 1
        END
    END
GO

-- ----------------------------------------------------------------------------------
-- Das Spielfeld darstellen. Dazu muss per "roten SQL" (dynamisches SQL) zur Laufzeit
-- je nach Abmessung des Spielbrettes selbiges gezeichnet werden
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcZeichnen]
AS
BEGIN
    DECLARE @sql VARCHAR(MAX)
    SELECT  @sql = ISNULL(@sql + ', ' + CHAR(13) + CHAR(9), 'SELECT ')
            + 'MAX(CASE WHEN [YKoord] = ' + CONVERT(VARCHAR(3), [YKoord])
            + ' THEN REPLACE([TSQLMine].[Spielfeld].[Status], ''0'', '' '') END) as [' + RIGHT('00' + CONVERT(VARCHAR(3), [YKoord]), 2)
            + ']'
    FROM [TSQLMine].[Spielfeld]
    GROUP BY [YKoord]
    ORDER BY [YKoord]
    SET @sql = @sql + CHAR(13) + 'FROM [TSQLMine].[Spielfeld]
    GROUP BY [XKoord]
    ORDER BY [XKoord]'
    EXEC (@sql)
END
GO

-- ----------------------------------------------------------------------------------
-- Die Lsung darstellen. Dazu muss per "roten SQL" (dynamisches SQL) zur Laufzeit
-- je nach Abmessung des Spielbrettes selbiges gezeichnet werden und die Minen werden
-- werden dabei aufgedeckt.
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcLoesungDarstellen]
AS
BEGIN
    DECLARE @sql VARCHAR(MAX)
    SELECT  @sql = ISNULL(@sql + ', ' + CHAR(13) + CHAR(9), 'SELECT ')
            + 'MAX(CASE WHEN [YKoord] = ' + CONVERT(VARCHAR(3), [YKoord])
            + ' THEN REPLACE(
                                CASE
                                                WHEN [TSQLMine].[Spielfeld].[Inhalt] IN (''M'') THEN ''M''
                                                WHEN [TSQLMine].[Spielfeld].[Inhalt] IN (''9'') THEN N''9''
                                                ELSE [TSQLMine].[Spielfeld].[Status]
                                END
                                , ''0'', '' '') END) as [' + RIGHT('00' + CONVERT(VARCHAR(3), [YKoord]), 2)
            + ']'
    FROM    [TSQLMine].[Spielfeld]
    GROUP BY [YKoord]
    ORDER BY [YKoord]
    SET @sql = @sql + CHAR(13) + 'FROM [TSQLMine].[Spielfeld]
    GROUP BY [XKoord]
    ORDER BY [XKoord]'
    EXEC (@sql)
END
GO
  
-- ----------------------------------------------------------------------------------
-- Es wird eine Statistikleiste ausgegeben, die den aktuellen Spielstatus auflistet
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcSpielstatus]
AS
BEGIN
	DECLARE @AnzahlMinen	INT
	DECLARE @XMax			INT
	DECLARE @YMax			INT
     
	SELECT    @AnzahlMinen	= AnzahlMinen
			, @XMax			= [XMax]
			, @YMax			= [YMax]
	FROM [TSQLMine].[Konfiguration]
     
	DECLARE @UntersuchteFelder		INT
	DECLARE @ExplodierteMinen		INT
	DECLARE @MarkierteFelder		INT

    SELECT @UntersuchteFelder = SUM(CASE 
										WHEN [Status] BETWEEN '0' AND '8' THEN 1
										ELSE 0
									END),
									@ExplodierteMinen = SUM(CASE 
										WHEN [Status] = '9' THEN 1
										ELSE 0
									END),
									@MarkierteFelder = SUM(CASE
										WHEN [Status] = 'X' THEN 1
										ELSE 0
									END)
    FROM [TSQLMine].[Spielfeld]
       
    SELECT  
		  @UntersuchteFelder		AS [UntersuchteFelder]
		, @ExplodierteMinen			AS [ExplodierteMinen]
		, @MarkierteFelder			AS [MarkierteFelder]
		, @AnzahlMinen				AS [AnzahlMinen]
		, @XMax * @YMax				AS [TotalSquares]
		, CASE 
            WHEN	1 = 1
					AND @ExplodierteMinen > 0
				THEN NCHAR(9760) + ' leider verloren...'
            WHEN	1 = 1
					AND @UntersuchteFelder + @AnzahlMinen = @XMax * @YMax
					AND @AnzahlMinen = @MarkierteFelder
					AND 
						(
							SELECT COUNT(*)
							FROM [TSQLMine].[Spielfeld]
							WHERE	1 = 1
									AND [Status] = 'X'
									AND [Inhalt] = 'M'
						) = @AnzahlMinen
                THEN 'Glckwunsch... gewonnen!'
            ELSE 'bitte weiterspielen!'
        END							AS [SpielStatus]     
END
GO

-- ----------------------------------------------------------------------------------
-- Diese Prozedur beschiesst ein einzelnes gegebenes Feld
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcSchuss] (@XKoord INT, @YKoord INT)
AS
BEGIN
    EXEC [TSQLMine].[prcEinzelfeld] @XKoord, @YKoord
    EXEC [TSQLMine].[prcLeerefelderAbraeumen]
    EXEC [TSQLMine].[prcZeichnen]
    EXEC [TSQLMine].[prcSpielstatus]
END
GO

-- ----------------------------------------------------------------------------------
-- Diese Prozedur markiert ein einzelnes gegebenes Feld
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcMarkierung] (@XKoord INT, @YKoord INT)
AS
BEGIN
    UPDATE [TSQLMine].[Spielfeld]
    SET [Status] = 'X'
    WHERE	1 = 1
			AND [XKoord] = @XKoord
			AND [YKoord] = @YKoord
			AND [Status] = '?'

    EXEC [TSQLMine].[prcLeerefelderAbraeumen]
    EXEC [TSQLMine].[prcZeichnen]
    EXEC [TSQLMine].[prcSpielstatus]
END
GO


---- ----------------------------------------------------------------------------------
---- Ein einzelnes Feld aufdecken
---- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcEinzelfeld] 
    (
          @XKoord INT
        , @YKoord INT
    )
AS
BEGIN
    IF                                                                                                                                                                                                         -- noch unbeschossenes Feld?
	(
		SELECT [Status] 
		FROM [TSQLMine].[Spielfeld]
		WHERE   1 = 1
				AND @XKoord	= [XKoord]
				AND @YKoord	= [YKoord]
	) = '?'
    BEGIN
        IF                                                                                                                                                                                                         -- Gibt es dort eine Mine?
        (
            SELECT COUNT(*)
            FROM [TSQLMine].[Spielfeld]
            WHERE   1 = 1
                    AND @XKoord		= [XKoord]
                    AND @YKoord		= [YKoord]
                    AND [Inhalt]	= 'M'
        ) > 0
        BEGIN
            UPDATE [TSQLMine].[Spielfeld]
            SET [Status] = '9'
            WHERE   1 = 1
                    AND @XKoord		= [XKoord]
                    AND @YKoord		= [YKoord]
        END
        ELSE
        BEGIN
			UPDATE [TSQLMine].[Spielfeld]
			SET [Status] = 
					(
						ISNULL(
							(
								SELECT COUNT(*) AS [GefundeneMinen]
								FROM   [TSQLMine].[Spielfeld] AS [SF]
								INNER JOIN [TSQLMine].[Spielfeld] AS [SFinnen] ON	1 = 1
																					AND [SF].[XKoord] - [SFinnen].[XKoord] BETWEEN -1 AND 1	-- Nachbarfelder ber die Koordinate ermitteln
																					AND [SF].[YKoord] - [SFinnen].[YKoord] BETWEEN -1 AND 1	-- Nachbarfelder ber die Koordinate ermitteln
								WHERE	1 = 1
										AND [SFinnen].[Inhalt]	= 'M'
										AND [SF].[XKoord]		= @XKoord
										AND [SF].[YKoord]		= @YKoord
								GROUP BY 
									  [SF].[XKoord]
									, [SF].[YKoord]
							)
						, 0)
					)
						WHERE   1 = 1
								AND @XKoord		= [XKoord]
								AND @YKoord		= [YKoord]
		END
	END
END
GO

-- ----------------------------------------------------------------------------------
-- Die Prozedur [TSQLMine].[prcLeerfelderAbraeumen] lst alle Felder auf, die sofort
-- aufgedeckt werden knnen. Solange es ein "0"-Feld gibt, welches noch von mindestens 
-- einem "?" umgeben ist, soll diese Routine weitere Felder aufdecken.
-- ----------------------------------------------------------------------------------
CREATE OR ALTER PROCEDURE [TSQLMine].[prcLeerefelderAbraeumen]
AS
BEGIN
    UPDATE [TSQLMine].[Spielfeld]
    SET [Status] = [Inhalt]
    FROM [TSQLMine].[Spielfeld] AS [aussen]
    WHERE	1 = 1
			AND [Status] = '?'
			AND ([Inhalt] <> 'M' OR [Inhalt] IS NULL)
			AND 
				(
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- oben drber
						WHERE [innen].[XKoord]		= [aussen].[XKoord]		AND [innen].[YKoord] + 1    = [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- oben rechts
						WHERE [innen].[XKoord] + 1	= [aussen].[XKoord]		AND [innen].[YKoord] + 1	= [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- rechts daneben
						WHERE [innen].[XKoord] + 1	= [aussen].[XKoord]		AND [innen].[YKoord]		= [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- unten rechts 
						WHERE [innen].[XKoord] + 1	= [aussen].[XKoord]		AND [innen].[YKoord] - 1	= [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- rechts drunter
						WHERE [innen].[XKoord]		= [aussen].[XKoord]		AND [innen].[YKoord] - 1	= [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- unten links
						WHERE [innen].[XKoord] - 1	= [aussen].[XKoord]		AND [innen].[YKoord] - 1	= [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- links daneben
						WHERE [innen].[XKoord] - 1	= [aussen].[XKoord]		AND [innen].[YKoord]		= [aussen].[YKoord]) = '0'
					OR
					(SELECT [Inhalt] FROM [TSQLMine].[Spielfeld] AS [innen]                                            -- links oben
						WHERE [innen].[XKoord] - 1	= [aussen].[XKoord]		AND [innen].[YKoord] + 1	= [aussen].[YKoord]) = '0'
				)
END
GO


-- ----------------------------------------------------------------------------------
-- Das Spiel starten..
-- Es wird eine Initialisierungsphase gestartet, die zuvor konfiguriert wird.
-- ----------------------------------------------------------------------------------
-- Anschlieend kann ber die Prozeduren 
-- 	   [TSQLMine].[prcSchuss]				--> einen Schuss auf eine Koordinate abgeben
-- 	   [TSQLMine].[prcMarkierung]			--> eine Koordinate als Minnevrdacht markieren
-- 	   [TSQLMine].[prcLoesungDarstellen]	--> eine Koordinate als Minnevrdacht markieren
-- ----------------------------------------------------------------------------------
DECLARE @ZufallsSeed	INTEGER
DECLARE @XAusdehnung	INTEGER
DECLARE @YAusdehnung	INTEGER
DECLARE @AnzahlMinen	INTEGER

SET @ZufallsSeed = (SELECT ABS(CONVERT(INT, CONVERT(binary(4), NEWID()))) % (32000 - 1) + 1)		-- ganzzahlige Zufallszahl zwischen 1 und 32.000
SET @XAusdehnung = 10																				-- Breite des Spielfeldes
SET @YAusdehnung = 20																				-- Hhe des Spielfeldes
SET @AnzahlMinen = 17																				-- Gesamtzahl der Minen

-- Initialisierung
EXEC [TSQLMine].[prcInitialisierung] @XAusdehnung, @YAusdehnung, @AnzahlMinen, @ZufallsSeed
EXEC [TSQLMine].[prcZeichnen]

-- eigentliches Spiel
--EXEC [TSQLMine].[prcSchuss] 7,6
--EXEC [TSQLMine].[prcMarkierung] 10,20
--EXEC [TSQLMine].[prcLoesungDarstellen]

