-- #####################################################################################
-- ## Skript zum Vortrag <die schwierigen Kopfnsse mit T-SQL gelst>                 ##     
-- #####################################################################################
-- ## Dieses Skript legt die Tabellen fr die Aufgabe RubiksCube an. Es wird durch    ##
-- ## weitere Skripte ergnzt, die in ihrer durchnummerierten Reihenfolge abzu-       ##
-- ## arbeiten sind. Die Skripte passen zu den Ausfhrungen aus der zugehrigen       ##
-- ## Powerpoint-Prsentation vom selben Autor.                                       ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Bitte dieses Skript niemals in einer Produktionsumgebung einsetzen, da          ##
-- ## Datenbankobjekte angelegt, ergnzt, gendert oder sogar gelscht werden!!!      ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Verwendung auf eigene Gefahr...                                                 ##
-- ## ------------------------------------------------------------------------------- ##
-- ## Torsten Ahlemeyer, arelium GmbH        torsten.ahlemeyer@arelium.de             ##
-- #####################################################################################

-- auf die Projektdatenbank wechseln
USE [Workshop_Kopfnuesse]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO


-----------------------------
-- Aufrumarbeiten ----------
-----------------------------
							 
DROP TABLE IF EXISTS [RubiksCube].[dreht_nach]
GO

DROP TABLE IF EXISTS [RubiksCube].[Position]
GO





-----------------------------
-- Aufbauarbeiten -----------
-----------------------------

-- Diese Tabelle stellt die Knoten des Graphen dar
CREATE TABLE [RubiksCube].[Position] (
	  [ID]              BIGINT       PRIMARY KEY
	, [PositionID]      VARCHAR(24)
	) AS NODE;
GO

-- Diese Tabelle stellt die Kanten, also die Verbindungen zwischen den Knoten, dar
CREATE TABLE [RubiksCube].[dreht_nach] (
	  [ID]              BIGINT       PRIMARY KEY 
    , [Drehung]         VARCHAR(25)
	, [Kosten]			INTEGER
	, [Startposition]	CHAR(24)
	, [Endposition]		CHAR(24)
    ) AS EDGE;
GO




-- Microsoft hat die SYS.TABLES-View angepasst und um entsprechende Kenner fr Graphen erweitert
SELECT 
         schema_name(schema_id) AS [Schema]
         , name
       , is_node
       , is_edge
FROM sys.tables
WHERE 1=1
       AND (name = 'dreht_nach' OR name = 'Position');
GO


-- Inhalte fr die Knotentabelle [RubiksCube].[Position] erzeugen
INSERT INTO [RubiksCube].[Position]
SELECT * FROM 
(
	SELECT 
		  ROW_NUMBER() OVER(ORDER BY [Ausgangsposition] ASC) - 1	AS [ID]							-- (Informatiker zhlen ab "0", daher "-1", da ROW_NUMBER bei "1" beginnt
		, [Ausgangsposition]										AS [PositionID]
	FROM
	(
		SELECT DISTINCT 
			[Ausgangsposition]										AS [Ausgangsposition]
		FROM [RubiksCube].[ZugKombinationen]
	) AS [DummyInnen]
) AS [DummyAussen]
GO


-- Inhalte fr die Kantentabelle [RubiksCube].[dreht_nach] erzeugen
INSERT INTO	[RubiksCube].[dreht_nach]
SELECT 
	  (SELECT $node_id FROM [RubiksCube].[Position] WHERE ID = [PO1].[ID])							-- KEIN Alias !!!
	, (SELECT $node_id FROM [RubiksCube].[Position] WHERE ID = [PO2].[ID])							-- KEIN Alias !!!
	, ROW_NUMBER() OVER(ORDER BY [Ausgangsposition] ASC) - 1					AS [ID]				-- (Informatiker zhlen ab "0", daher "-1", da ROW_NUMBER bei "1" beginnt	
	, [Bewegung]																AS [Drehung]		-- Identifiziert die Drehung, z.B. "RechtsHoch"
	, 1																			AS [Kosten]			-- Zugzahl zur Berechnung der "Gotteszahl" (Anzahl minimaler Zge bis zur Lsung)
	, [Ausgangsposition]														AS [Startposition]	-- Position VOR dem Drehen
	, [Zielposition]															AS [Endposition]	-- Position NACH dem Drehen
FROM [RubiksCube].[ZugKombinationen] AS [ZK]
INNER JOIN [RubiksCube].[Position] AS [PO1]
	ON 1 = 1
	AND [ZK].[Ausgangsposition] = [PO1].[PositionID]
INNER JOIN [RubiksCube].[Position] AS [PO2]
	ON 1 = 1
	AND [ZK].[Zielposition] = [PO2].[PositionID]
GO





-- der SQL Server legt zustzliche Spalten an! Diese sind VERSTECKT
SELECT object_id, name, is_hidden, graph_type, graph_type_desc FROM     SYS.COLUMNS WHERE graph_type_desc IS NOT NULL
GO
-- der SQL Server legt selber passende Indizes an

